import NodeTableComponent, {
  DataTemplateInterface,
} from "../NodeTableComponent/NodeTableComponent";
import { useEffect, useState } from "react";
import { invoke } from "@tauri-apps/api/core";
import styles from "./SectionHeaders.module.scss";
import { Flex } from "antd";
const dataTemplate: DataTemplateInterface = {
  name: {
    name: "name",
    offset: 0,
    size: 8,
    data_type: "char_array",
    description: "节区的名称，一般以.开头",
  },
  virtual_size: {
    name: "virtual_size",
    offset: 8,
    size: 4,
    data_type: "hex",
    description: "在内存中的大小,不能为0",
  },
  virtual_address: {
    name: "virtual_address",
    offset: 12,
    size: 4,
    data_type: "hex",
    description: "在内存中的地址(VA)",
  },
  size_of_raw_data: {
    name: "size_of_raw_data",
    offset: 16,
    size: 4,
    data_type: "hex",
    description: "在文件中的大小",
  },
  pointer_to_raw_data: {
    name: "pointer_to_raw_data",
    offset: 20,
    size: 4,
    data_type: "hex",
    description: "在文件中的偏移",
  },
  pointer_to_relocations: {
    name: "pointer_to_relocations",
    offset: 24,
    size: 4,
    data_type: "hex",
    description: "重定位表的偏移",
  },
  pointer_to_linenumbers: {
    name: "pointer_to_linenumbers",
    offset: 28,
    size: 4,
    data_type: "hex",
    description: "行号表的偏移",
  },
  number_of_relocations: {
    name: "number_of_relocations",
    offset: 32,
    size: 2,
    data_type: "hex",
    description: "重定位表的数量",
  },
  number_of_linenumbers: {
    name: "number_of_linenumbers",
    offset: 34,
    size: 2,
    data_type: "hex",
    description: "行号表的数量",
  },
  characteristics: {
    name: "characteristics",
    offset: 36,
    size: 4,
    data_type: "hex",
    description: "属性",
  },
};
// 节区头大小
const sectionHeaderSize = 40;

export default function SectionHeaders() {
  const [data, setData] = useState([]);

  const command = "command_get_pe_data_section_headers";

  const columns = [
    {
      title: "字段名",
      dataIndex: "name",
      key: "name",
    },
    {
      title: "偏移",
      dataIndex: "offset",
      key: "offset",
      hexSwitch: true,
    },
    {
      title: "大小",
      dataIndex: "size",
      key: "size",
    },
    {
      title: "值",
      dataIndex: "value",
      key: "value",
      hexSwitch: true,
      editable: true,
    },
    {
      title: "描述信息",
      dataIndex: "description",
      key: "description",
    },
  ];

  const formatDataHandler = (data: any) => {
    const { base_offset } = data;
    return (
      data?.fields?.map((item: any, index: number) => {
        return {
          base_offset: base_offset + index * sectionHeaderSize,
          fields: { ...item },
        };
      }) || []
    );
  };

  useEffect(() => {
    invoke(command).then((res) => {
      setData(formatDataHandler(res));
    });
  }, []);
  return (
    <Flex className={styles.content}>
      {data.map((item, index) => {
        return (
          <div className={styles.tableItem}>
            <NodeTableComponent
              key={index}
              dataTemplate={dataTemplate}
              defaultData={item}
              columns={columns as any}
            />
          </div>
        );
      })}
    </Flex>
  );
}
